package com.example.angel.metrosalud


import android.content.Intent
import android.os.Bundle
import android.view.View
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import com.example.angel.metrosalud.adapters.SafeClickListener
import com.example.angel.metrosalud.data.Alumno
import com.example.angel.metrosalud.databinding.ActivityMainBinding
import com.example.angel.metrosalud.myFirebase.MyCallback
import com.example.angel.metrosalud.myFirebase.MyFirebaseAPI
import com.example.angel.metrosalud.myUI.LoadingDialog
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.FirebaseUser
import com.google.firebase.auth.ktx.auth
import com.google.firebase.database.DatabaseReference
import com.google.firebase.database.FirebaseDatabase
import com.google.firebase.ktx.Firebase


class MainActivity : AppCompatActivity() {

    private lateinit var binding: ActivityMainBinding

    private lateinit var auth: FirebaseAuth
    private lateinit var databaseReference: DatabaseReference
    private lateinit var database: FirebaseDatabase
    private lateinit var loading: LoadingDialog

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityMainBinding.inflate(layoutInflater)
        loading = LoadingDialog(this)
        setContentView(binding.root)
        initialise()
    }

    private fun initialise() {
        auth = Firebase.auth
        database = FirebaseDatabase.getInstance()
        databaseReference = database.reference
        binding.buttonRegister.setSafeOnClickListener {
            val intent = Intent(this, RegisterActivity::class.java)
            startActivity(intent)
        }
        binding.buttonLogIn.setSafeOnClickListener {
            login()
        }
        binding.tvPasswordReset.setSafeOnClickListener {
            if (checkEmail(binding.editTextEmail.text.toString())){
                loading.startLoading()
                MyFirebaseAPI(this).sendPasswordEmail(binding.editTextEmail.text.toString(), object: MyCallback{
                    override fun onSendPasswordEmail(b: Boolean){
                        loading.isDismiss()
                        if (b){
                            val dialog = AlertDialog.Builder(this@MainActivity)
                                .setTitle("Reset Password")
                                .setMessage("Se ha enviado un email a la dirección: ${binding.editTextEmail.text}." +
                                        " Por favor siga los pasos para restablecer su contraseña")
                                .setPositiveButton("OK"){ view, _ ->
                                    view.dismiss()
                                }
                                .setCancelable(false)
                                .create()
                            dialog.show()
                        }else{
                            val dialog = AlertDialog.Builder(this@MainActivity)
                                .setTitle("Reset Password")
                                .setMessage("No se encuentra la dirección: ${binding.editTextEmail.text}." +
                                        "Por favor introduzca una dirección registrada.")
                                .setPositiveButton("OK"){ view, _ ->
                                    view.dismiss()
                                }
                                .setCancelable(false)
                                .create()
                            dialog.show()
                        }

                    }
                })
            }else{
                Toast.makeText(baseContext, "Introduce una dirección de email válida", Toast.LENGTH_SHORT).show()
            }
        }
    }


    private fun login() {
        /**Check email and password are not empty*/
        if (binding.editTextEmail.text.toString().isNotEmpty()
                && binding.editTextPassword.text.toString().isNotEmpty()
        ) {
            if(checkEmail(binding.editTextEmail.text.toString())){
                loading.startLoading()
                auth.signInWithEmailAndPassword(
                    binding.editTextEmail.text.toString(),
                    binding.editTextPassword.text.toString()
                )
                    .addOnCompleteListener(this) { task ->
                        if (task.isSuccessful) {
                            /**if successful sign in check user type*/
                            val user: FirebaseUser = auth.currentUser
                            checkUserTypeAndChangeActivity(user)
                        } else {
                            /**if not succesful dismiss loading dialog and show error message*/
                            loading.isDismiss()
                            Toast.makeText(baseContext, "Se ha producido un error", Toast.LENGTH_SHORT).show()
                        }
                    }
            }else{
                Toast.makeText(baseContext, R.string.check_email_is_valid, Toast.LENGTH_SHORT)
                    .show()
            }

        } else {
            Toast.makeText(baseContext, R.string.check_email_pass, Toast.LENGTH_SHORT)
                .show()
        }
    }

    private fun checkEmail(email: String): Boolean{
        var emailPattern = "[a-zA-Z0-9._-]+@[a-z]+\\.+[a-z]+"
        return email.trim { it <= ' ' }.matches(emailPattern.toRegex())
    }

    private fun checkUserTypeAndChangeActivity(mUser: FirebaseUser) {
        /**Check if user is teacher or student and change to proper activity*/
        databaseReference.child("Profesores").child(mUser.uid).get().addOnSuccessListener {
            loading.isDismiss()
            if (it.value != null) {
                val intent = Intent(this, TeacherInitActivity::class.java)
                finish()
                startActivity(intent)
            } else {
                /**Check if user has group or not */
                MyFirebaseAPI(this).getStudent(object : MyCallback {
                    override fun onStudent(alu: Alumno) {
                        if(alu.idGrupo != "-1"){
                            val intent = Intent(this@MainActivity,StudentProfileActivity::class.java)
                            intent.putExtra("alu",alu)
                            finish()
                            startActivity(intent)
                        }else{
                            val intent = Intent(this@MainActivity, StudentEmptyActivity::class.java)
                            finish()
                            startActivity(intent)
                        }
                    }

                })
            }
        }.addOnFailureListener {
            /**Error while connecting to database. Dismiss loading dialog and show message*/
            loading.isDismiss()
            Toast.makeText(baseContext, "Se ha producido un error con la conexión a la Base de Datos",
                Toast.LENGTH_SHORT).show()
        }
    }

    /*override fun onStart() {
        super.onStart()
        if (MyFirebaseAPI(this).getCurrentUser()!=null)
            MyFirebaseAPI(this).logOut()
    }*/

    private fun View.setSafeOnClickListener(onSafeClick: (View) -> Unit) {
        val safeClickListener = SafeClickListener {
            onSafeClick(it)
        }
        setOnClickListener(safeClickListener)
    }

}