package com.example.angel.metrosalud.myFirebase

import android.app.Activity
import android.renderscript.Sampler
import android.util.Log

import com.example.angel.metrosalud.data.*
import com.example.angel.metrosalud.myUI.LoadingDialog
import com.fasterxml.jackson.databind.node.BooleanNode
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.FirebaseUser
import com.google.firebase.auth.ktx.auth
import com.google.firebase.database.*
import com.google.firebase.database.core.Context
import com.google.firebase.ktx.Firebase

class MyFirebaseAPI(val activity: Activity) {
    private var auth: FirebaseAuth = Firebase.auth
    private var database: FirebaseDatabase = FirebaseDatabase.getInstance()
    private var databaseReference: DatabaseReference = database.reference
    private var loading: LoadingDialog = LoadingDialog(activity)


    /** Function that adds a group to Firebase Database
     * Input: group object
     * */
    fun addGroup(myGroup: Group, myCallback: MyCallback) {
        myGroup.idProfesor = getCurrentUser()?.uid
        val ref = databaseReference.child("Grupos")
        val myKey: String? = ref.push().key
        myGroup.id = myKey
        val myDBRef = myKey?.let { ref.child(it) }
        loading.startLoading()
        myDBRef?.addListenerForSingleValueEvent(object : ValueEventListener {
            override fun onDataChange(snapshot: DataSnapshot) {
                loading.isDismiss()
                myCallback.onAddGroup(true)
            }

            override fun onCancelled(error: DatabaseError) {
                loading.isDismiss()
                myCallback.onAddGroup(false)
            }
        })
        myDBRef?.setValue(myGroup)
    }

    /** Function that removes a group from Firebase Database
     * Input: idGroup
     * */
    fun removeGroup(idGroup: String, myCallback: MyCallback){
        databaseReference.child("Grupos").child(idGroup).removeValue()
            .addOnSuccessListener {
            myCallback.onRemoveGroup(true)
            }
            .addOnFailureListener {
            myCallback.onRemoveGroup(false)
            }
    }

    /** Check enrolled students to a group
     * Used when teacher removes a group to unsubscribe students
     * Input: idGroup
     * */
    fun unsubscribeStudents(idGroup: String, myCallback: MyCallback){
        databaseReference.child("Alumnos").orderByChild("idGrupo").equalTo(idGroup)
            .addListenerForSingleValueEvent(object : ValueEventListener{
                override fun onDataChange(snapshot: DataSnapshot) {
                    if (snapshot.hasChildren()){
                        snapshot.children.forEach {
                            it.key?.let { it1 -> databaseReference.child("Alumnos").child(it1).child("idGrupo").setValue("-1") }
                        }
                    }
                    myCallback.onUnsubscribeStudents(true)
                }
                override fun onCancelled(error: DatabaseError) {
                }

            })
    }

    /** Function that adds a student to Firebase Database
     * Input: alumno object
     * */
    fun addStudent(alumno: Alumno, myCallback: MyCallback) {
        val myDBRef = getCurrentUser()?.uid?.let { databaseReference.child("Alumnos").child(it) }
        loading.startLoading()
        myDBRef?.addListenerForSingleValueEvent(object : ValueEventListener {
            override fun onDataChange(snapshot: DataSnapshot) {
                myCallback.onAddStudent(true)
            }

            override fun onCancelled(error: DatabaseError) {
                loading.isDismiss()
                myCallback.onAddStudent(false)
            }
        })
        myDBRef?.setValue(alumno)
    }

    /** Function that updates a student
     * Input: alumno
     * */
    fun updateStudent(alumno: Alumno, myCallback: MyCallback){
        val myDBRef = getCurrentUser()?.let { databaseReference.child("Alumnos").child(it.uid) }
        myDBRef?.addListenerForSingleValueEvent(object : ValueEventListener{
            override fun onDataChange(snapshot: DataSnapshot) {
                myCallback.onUpdateStudent(true)
            }

            override fun onCancelled(error: DatabaseError) {
                myCallback.onUpdateStudent(false)
            }

        })
        myDBRef?.setValue(alumno)
    }

    /** Function that sets student points to zero. When student exits from a group
     *
     * */
    fun setZeroPointsOnStudent(myCallback: MyCallback){
        val myDBRef = database.getReference("Alumnos").child(auth.currentUser.uid).child("puntos")
        myDBRef?.addListenerForSingleValueEvent(object : ValueEventListener{
            override fun onDataChange(snapshot: DataSnapshot) {
                myCallback.onSetZeroPointsOnStudent(true)
            }

            override fun onCancelled(error: DatabaseError) {
                myCallback.onSetZeroPointsOnStudent(false)
            }

        })
        myDBRef?.setValue("0")
    }

    fun deletePoints(idGroup: String, myCallback: MyCallback){
        val myDBRef = database.getReference("Puntos").child(auth.currentUser.uid).child(idGroup).removeValue()
            .addOnSuccessListener {
                myCallback.onDeletePoints(true)
            }
            .addOnFailureListener {
                myCallback.onDeletePoints(false)
            }
    }


    /** Function that adds a Teacher to Firebase Database
     * Input: profesor object
     * */
    fun addTeacher(profesor: Profesor, myCallback: MyCallback) {
        val myDBRef = getCurrentUser()?.let { databaseReference.child("Profesores").child(it.uid) }
        loading.startLoading()
        myDBRef?.addListenerForSingleValueEvent(object : ValueEventListener {
            override fun onDataChange(snapshot: DataSnapshot) {
                myCallback.onAddTeacher(true)
            }

            override fun onCancelled(error: DatabaseError) {
                loading.isDismiss()
                myCallback.onAddTeacher(false)
            }
        })
        myDBRef?.setValue(profesor)
    }

    /** Function that gets current user logged in Firebase
     *
     */
    fun getCurrentUser(): FirebaseUser? {
        return auth.currentUser
    }


    /** Function that gets current student connected on Firebase Database
     * Output: alumno object
     * */
    fun getStudent(myCallback: MyCallback) {
        database.getReference("Alumnos")
            .child(auth.currentUser.uid)
            .addListenerForSingleValueEvent(object : ValueEventListener {
                override fun onDataChange(snapshot: DataSnapshot) {
                    val alumno: Alumno? = snapshot.getValue(Alumno::class.java)
                    if (alumno != null) {
                        myCallback.onStudent(alumno)
                    }
                }

                override fun onCancelled(error: DatabaseError) {
                    Log.d("FIREBASE", "Error leyendo alumno")
                }
            })
    }

    /** Function that gets idGroup of a Group
     * Input: inscription Code. Each Inscription code must be different
     * Output: idGroup of group with that inscription code
     * */
    fun getIdGroup(code: String, myCallback: MyCallback) {
        database.getReference("Grupos")
            .orderByChild("inscriptionCode")
            .equalTo(code)
            .addListenerForSingleValueEvent(object : ValueEventListener {
                override fun onDataChange(snapshot: DataSnapshot) {
                    val id: String? = snapshot.children.elementAt(0).key.toString()
                    if (id != null) {
                        myCallback.onIdGroup(id)
                    }
                }

                override fun onCancelled(error: DatabaseError) {
                    Log.d("FIREBASE-Grupo", "ERROR")
                }
            })
    }

    /** Function that gets max incription code of every group.
     * It is needed to calculate inscription code of new group
     * Output: Max Inscription Code
     */
    fun getMaxInscriptionCode(myCallback: MyCallback){
        database.getReference("Grupos").orderByChild("inscriptionCode").limitToLast(1)
            .addListenerForSingleValueEvent(object : ValueEventListener{
                override fun onDataChange(snapshot: DataSnapshot) {
                    val id: String? = snapshot.children.elementAt(0).getValue(Group::class.java)?.inscriptionCode
                    if (id != null){
                        myCallback.onGetMaxInscriptionCode(id)
                    }
                }
                override fun onCancelled(error: DatabaseError) {
                    Log.d("FIREBASE", "ERROR-getMaxInscriptionCode")
                }

            })
    }

    /** Function that gets number of stations of a group
     * It is needed to create points object
     * Input: idGroup
     * Ouput: number of stations
     * */
    fun getStationSize(idGroup: String, myCallback: MyCallback){
        database.getReference("Grupos").child(idGroup).child("parada")
            .addListenerForSingleValueEvent(object : ValueEventListener{
                override fun onDataChange(snapshot: DataSnapshot) {
                    val size: Int = snapshot.children.count()
                    myCallback.onStationSize(size)
            }

                override fun onCancelled(error: DatabaseError) {
            }

        })
    }

    /**Function that enrolls student to a group
     * It changes idGroup on Student Object
     * Input: idGroup
     * Output: true if changed
     * */
    fun updateIdGroupOnStudent(id: String, myCallback: MyCallback){
        val myDBRef = database.getReference("Alumnos").child(auth.currentUser.uid).child("idGrupo")
        myDBRef.addListenerForSingleValueEvent(object : ValueEventListener{
            override fun onDataChange(snapshot: DataSnapshot) {
                myCallback.onUpdateIdGroupOnStudent(true)
            }

            override fun onCancelled(error: DatabaseError) {
                myCallback.onUpdateIdGroupOnStudent(false)
            }
        })
        myDBRef?.setValue(id)
    }

    /** Function that adds Points Object on Firebase RealTime Database
     * Input: Puntos object, idGroup
     * Output: true if added
     */
    fun addPoints(points: Puntos, idGroup: String, myCallback: MyCallback) {
        val myDBRef = database.getReference("Puntos").child(auth.currentUser.uid).child(idGroup)
        myDBRef.addListenerForSingleValueEvent(object : ValueEventListener{
            override fun onDataChange(snapshot: DataSnapshot) {
                myCallback.onAddPoints(true)
            }

            override fun onCancelled(error: DatabaseError) {
                myCallback.onAddPoints(false)
            }

        })
        myDBRef.setValue(points)
    }

    /** Function that updates points on Firebase. It updates an station points.
     * Input: achievement to update. Index to identify station. IdGroup.
     * NewTotalPoints previously calculated
     * */
    fun updatePoints(achievement: Achievement, index: Int, idGroup: String, newTotalPoints: Int, myCallback: MyCallback){
        val myDBRef = database.getReference("Puntos").child(auth.currentUser.uid).child(idGroup).child("puntos").child(index.toString())
        myDBRef.addListenerForSingleValueEvent(object : ValueEventListener{
            override fun onDataChange(snapshot: DataSnapshot) {
                updateTotalPoints(idGroup,newTotalPoints.toString(), myCallback)
            }

            override fun onCancelled(error: DatabaseError) {
            }

        })
        myDBRef.setValue(achievement)
    }

    /** Function that updates total points in Points object and also in Alumno Object
     * Input: idGroup, new points
     * */
    fun updateTotalPoints(idGroup: String,points: String, myCallback: MyCallback){
        val myDBRef = database.getReference("Puntos").child(auth.currentUser.uid).child(idGroup).child("pointsSum")
        myDBRef.addListenerForSingleValueEvent(object : ValueEventListener{
            override fun onDataChange(snapshot: DataSnapshot) {
                myCallback.onUpdatePointsAndTotalPoints(true)
            }

            override fun onCancelled(error: DatabaseError) {
            }

        })
        myDBRef.setValue(points)
        //Update also totalPoints in Alumno object.
        //No need to capture event
        val myDBRef2 = database.getReference("Alumnos").child(auth.currentUser.uid).child("puntos")
        myDBRef2.setValue(points)

    }

    /**Function that gets an achievement
     * Input: idGroup
     * Output: Achievements
     */
    fun getAchievement(idGroup: String, position: Int, myCallback: MyCallback){
        val myDBRef = database. getReference("Puntos").child(auth.currentUser.uid).child(idGroup)
            myDBRef.addListenerForSingleValueEvent(object : ValueEventListener{
                override fun onDataChange(snapshot: DataSnapshot) {
                    val puntos = snapshot.getValue(Puntos::class.java)
                    if (puntos != null) {
                        puntos.puntos?.get(position)?.let { myCallback.onAchievement(it) }
                         }
                    }
                override fun onCancelled(error: DatabaseError) {
                }

            })
    }

    /** Function that gets all groups of a teacher
     * */
    fun getGroups( myCallback: MyCallback) {
        database.getReference("Grupos").orderByChild("idProfesor").equalTo(getCurrentUser()?.uid).
            addListenerForSingleValueEvent(object: ValueEventListener{
                override fun onDataChange(snapshot: DataSnapshot) {
                    var list = arrayListOf<Group>()
                    if (snapshot.hasChildren()){
                        snapshot.children.forEach {
                            var g = it?.getValue(Group::class.java)
                            if (g != null) {
                                list.add(g)
                            }
                        }
                    }
                    myCallback.onGetGroups(list)
                }
                override fun onCancelled(error: DatabaseError) {
                }

            })
    }

    /** Function that gets all students enrolled to a group
     * Input: idGroup
     * */
    fun getStudents(idGroup: String, myCallback: MyCallback){
        database.getReference("Alumnos").orderByChild("idGrupo").equalTo(idGroup)
            .addListenerForSingleValueEvent(object: ValueEventListener{
                override fun onDataChange(snapshot: DataSnapshot) {
                   var list = arrayListOf<Alumno>()
                    if (snapshot.hasChildren()){
                        snapshot.children.forEach{
                            var a = it?.getValue(Alumno::class.java)
                            if (a != null) {
                                list.add(a)
                            }
                        }
                    }
                    myCallback.onGetStudents(list)
                }

                override fun onCancelled(error: DatabaseError) {
                }

            })
    }

    /** Function that gets all stations of a group
     * Input: idGroup
     */
    fun getStations(idGroup: String?, myCallback: MyCallback) {
        if (idGroup != null) {
            databaseReference.child("Grupos").child(idGroup).addListenerForSingleValueEvent(object : ValueEventListener{
                override fun onDataChange(snapshot: DataSnapshot) {
                    val grupo: Group? = snapshot.getValue(Group::class.java)
                    if (grupo != null) {
                        grupo.parada?.let { myCallback.onStations(it) }
                    }
                }

                override fun onCancelled(error: DatabaseError) {
                    Log.d("FIREBASE-getStations","Se ha producido un error leyendo las estaciones")
                }

            })
        }
    }

    /** Function that gets points of current student logged in Firebase
     * Input: idGroup
     * */
    fun getPoints(idGroup: String, myCallback: MyCallback){
        getCurrentUser()?.let {
            databaseReference.child("Puntos").child(it.uid).child(idGroup).addListenerForSingleValueEvent(
                object : ValueEventListener{
                    override fun onDataChange(snapshot: DataSnapshot) {
                        val puntos: Puntos? = snapshot.getValue(Puntos::class.java)
                        if (puntos != null){
                            puntos.puntos?.let { it1 -> myCallback.onGetPoints(it1) }
                        }
                    }

                    override fun onCancelled(error: DatabaseError) {
                    }

                }
            )
        }
    }

    /** Function that gets group name
     * Input: idGroup
     * */
    fun getGroupName(idGroup: String, myCallback: MyCallback){
        databaseReference.child("Grupos").child(idGroup).child("name")
            .addListenerForSingleValueEvent(object : ValueEventListener{
                override fun onDataChange(snapshot: DataSnapshot) {
                    if (snapshot != null){
                        myCallback.onGetGroupName(snapshot.value.toString())
                    }
                }

                override fun onCancelled(error: DatabaseError) {
                }
            }
        )
    }

    /** Function that gets total points of current student logged in Firebase
     * Input: idGroup
     * */
    fun getTotalPoints(idGroup: String, myCallback: MyCallback){
        getCurrentUser()?.let {
            databaseReference.child("Puntos").child(it.uid).child(idGroup).child("pointsSum")
            .addListenerForSingleValueEvent(object : ValueEventListener{
                override fun onDataChange(snapshot: DataSnapshot) {
                    if (snapshot != null){
                        myCallback.onGetTotalPoints(snapshot.value.toString())
                    }
                }

                override fun onCancelled(error: DatabaseError) {
                }

            })
        }
    }

    /**Function that get idGroup of current authenticated user
     * */
    fun getMyGroup(myCallback: MyCallback){
        getCurrentUser()?.let {
            databaseReference.child("Alumnos").child(it.uid).child("idGrupo")
            .addListenerForSingleValueEvent(object : ValueEventListener{
                override fun onDataChange(snapshot: DataSnapshot) {
                    if (snapshot != null){
                        myCallback.onGetMyGroup(snapshot.value.toString())
                    }
                }

                override fun onCancelled(error: DatabaseError) {
                }
            })
        }
    }

    /** Function that logs out current user
     * */
    fun logOut(){
        if (getCurrentUser() != null){
            FirebaseAuth.getInstance().signOut()
            //auth.signOut()
        }
    }

    /** Function that resets password of user
     * Input: email String
     * Output: true if correct
     */
    fun sendPasswordEmail(email: String, myCallback: MyCallback){
        Firebase.auth.sendPasswordResetEmail(email)
            .addOnCompleteListener { task ->
                if (task.isSuccessful) {
                    myCallback.onSendPasswordEmail(true)
                }else{
                    myCallback.onSendPasswordEmail(false)
                }
            }
    }
}

interface MyCallback {
    fun onStudent(alu: Alumno){}
    fun onIdGroup(id: String){}
    fun onUpdateIdGroupOnStudent(bool: Boolean){}
    fun onStations(list: MutableList<Parada>){}
    fun onStationSize(size: Int){}
    fun onAddPoints(b: Boolean){}
    fun onAchievement(achievement: Achievement){}
    fun onAddGroup(b: Boolean){}
    fun onAddStudent(b: Boolean){}
    fun onUpdateStudent(b: Boolean){}
    fun onAddTeacher(b: Boolean){}
    fun onGetMaxInscriptionCode(id: String){}
    fun onGetGroups(list: ArrayList<Group>){}
    fun onRemoveGroup(b: Boolean){}
    fun onGetStudents(list: ArrayList<Alumno>){}
    fun onGetPoints(list: MutableList<Achievement>){}
    fun onUnsubscribeStudents(b: Boolean){}
    fun onGetGroupName(str: String){}
    fun onGetTotalPoints(str: String){}
    fun onGetMyGroup(str: String){}
    fun onSendPasswordEmail(b: Boolean){}
    fun onGetActivities(b: Boolean){}
    fun onUpdatePointsAndTotalPoints(b: Boolean){}
    fun onSetZeroPointsOnStudent(b: Boolean){}
    fun onDeletePoints(b: Boolean){}
    fun onRemoveGroupFromStudent(b: Boolean){}
}















